@@ -187,7 +187,7 @@ def read_row(workSheet, Nrow, St = False):
     rowValues = rowValues[rowValues != None]
     return rowValues
 
-def CompareLEDETParameters(FileA, FileB, Precision = 1E-5):
+def CompareLEDETParameters(FileA, FileB, Precision = 1E-5, showIndices = 0):
     Diff = 0
 
     P_a = ParametersLEDET()
@@ -241,6 +241,7 @@ def CompareLEDETParameters(FileA, FileB, Precision = 1E-5):
                     print("Different Positions: {}".format(Pos))
                 else:
                     print("Many values are different (>10)")
+                    if showIndices: print(Pos)
 
 
     ## Check Options
