@@ -1,19 +1,20 @@
 import os
 import numpy as np
 import pandas as pd
+from steam_nb_api.ledet.ParametersLEDET import ParametersLEDET
+import csv
+from matplotlib import cm
+from mpl_toolkits.mplot3d import Axes3D
 import re
 from scipy.interpolate import interp1d
 from tqdm import tqdm
-from matplotlib import cm
-from mpl_toolkits.mplot3d import Axes3D
 import matplotlib.cm as cm
 from matplotlib.collections import PolyCollection
 import matplotlib.pyplot as plt
 from matplotlib import colors as mcolors
 import h5py
+import time
 from steam_nb_api.ledet.ParameterSweep import *
-from steam_nb_api.ledet.ParametersLEDET import ParametersLEDET
-import csv
 import nptdms
 from nptdms import TdmsFile
 
@@ -38,6 +39,7 @@ class TDMSdata:
     U_QH: np.ndarray = np.array([])
     QL1: float = 0.0
     QL2: float = 0.0
+    Quenches: np.ndarray = np.array([])
 
 @dataclass
 class SIMdata:
@@ -410,6 +412,7 @@ class QuenchPlanAnalysis():
     def __init__(self,FileNameTDMS, FileNameSIMData):
         self.beforePC = 0.1
         self.Colors = self.__generateColors()
+        self.verbose = False
 
         self.MergedVoltages = {}
 
@@ -421,7 +424,7 @@ class QuenchPlanAnalysis():
         self.TDMSdata.FileName = FileNameTDMS
         self.__CreateTDMSDataObject()
 
-        self.__calculateAllQuenchIntegrals()
+        # self.__calculateAllQuenchIntegrals()
 
     def __generateColors(self):
         # setting colors for plotting
@@ -522,7 +525,8 @@ class QuenchPlanAnalysis():
         trigger_shoot = np.where(abs(trigger_shoot)>self.TDMSdata.Tolerance)[0][0]-int(self.beforePC/self.TDMSdata.t_d_MF)
         self.TDMSdata.trigger_PC = int(self.beforePC/self.TDMSdata.t_d_MF)
         self.TDMSdata.trigger_shoot = trigger_shoot
-        self.TDMSdata.TimeFrame_MF = self.TDMSdata.TimeFrame_MF[trigger_shoot:]-self.TDMSdata.TimeFrame_MF[trigger_shoot]
+        self.TDMSdata.TimeFrame_MF = self.TDMSdata.TimeFrame_MF[trigger_shoot:]-self.TDMSdata.TimeFrame_MF[trigger_shoot]\
+                                     -self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_PC+1]
         self.TDMSdata.I_DCCT_MF = self.TDMSdata.I_DCCT_MF[trigger_shoot:]
         self.TDMSdata.I_CLIQ = self.TDMSdata.I_CLIQ[trigger_shoot:]
         self.TDMSdata.U_CLIQ = self.TDMSdata.U_CLIQ[trigger_shoot:]
@@ -542,10 +546,10 @@ class QuenchPlanAnalysis():
             if trigger < float(self.SIMdata.TimeFrame[0]):
                 self.beforePC = -1* self.SIMdata.TimeFrame[0]
                 trigger = np.array(f.get("t_PC"))[0][0] - self.beforePC
-                print("Switch aligning to tStart of Simulation!")
+                if self.verbose: print("Switch aligning to tStart of Simulation!")
             trigger_shoot = np.where(abs(self.SIMdata.TimeFrame-trigger)<1E-4)[0][0]
-            self.SIMdata.trigger_PC = np.where(abs(self.SIMdata.TimeFrame-np.array(f.get("t_PC"))[0][0])<1E-4)[0][0]
-            self.SIMdata.TimeFrame = self.SIMdata.TimeFrame[trigger_shoot:]-self.SIMdata.TimeFrame[trigger_shoot]
+            self.SIMdata.trigger_PC = np.where(abs(self.SIMdata.TimeFrame-np.array(f.get("t_PC"))[0][0])<1E-7)[0][0]
+            self.SIMdata.TimeFrame = self.SIMdata.TimeFrame[trigger_shoot:]
             self.SIMdata.I_CoilSections = self.SIMdata.I_CoilSections[:, trigger_shoot:]
             self.SIMdata.I_CLIQ = self.SIMdata.I_CLIQ[trigger_shoot:]
             self.SIMdata.directionsCLIQ = np.array(f.get("directionCurrentCLIQ"))
@@ -564,12 +568,36 @@ class QuenchPlanAnalysis():
         QL  = np.sum(np.multiply(dt, np.power(current,2)))/1E6
         return QL
 
-    def __calculateAllQuenchIntegrals(self):
-        ## QL Integral 1 - from Quench
-        ## Find quench
-        ## Cut parts and
+    def __FindQuenchSIM(self, tQuench):
+        try:
+            idxQuench = np.where(abs(self.SIMdata.TimeFrame - float(tQuench)) < 1E-7)[0][0]
+        except:
+            try:
+                idxQuench = np.where(abs(self.SIMdata.TimeFrame - float(tQuench)) < 1E-4)[0][0]
+            except:
+                if self.verbose: print("Quench time of TDMS not found in simulation time frame, t=", tQuench)
+                idxQuench = 0
+        return idxQuench
 
-        ## QL Integral 2  - full transient
+    def calculateAllQuenchIntegrals(self):
+        ## QL Integral 1 - from Quench
+        ## QL Measured Data
+        if not self.TDMSdata.Quenches.size > 0: self.FindQuenchTDMS(0.1, 0.007, Plot =0)
+        idxQuench_TDMS = int(self.TDMSdata.Quenches[0][2])
+        self.__FindQuenchSIM(self.TDMSdata.Quenches[0][1])
+        self.TDMSdata.QL1 = self.__calculateQuenchIntegral(self.TDMSdata.TimeFrame_MF[idxQuench_TDMS:],
+                                                           self.TDMSdata.I_DCCT_MF[idxQuench_TDMS:])
+        ## QL Sim Data
+        idxQuench_SIM = self.__FindQuenchSIM(self.TDMSdata.Quenches[0][1])
+        if abs(sum(self.SIMdata.I_CLIQ)) > 0:
+            idxC = np.where(self.SIMdata.directionsCLIQ > 0)[0][0]
+            self.SIMdata.QL1 = self.__calculateQuenchIntegral(self.SIMdata.TimeFrame[idxQuench_SIM:, 0],
+                                                              self.SIMdata.I_CoilSections[idxQuench_SIM:,
+                                                              idxC])
+        else:
+            self.SIMdata.QL1 = self.__calculateQuenchIntegral(self.SIMdata.TimeFrame[idxQuench_SIM:, 0],
+                                                              self.SIMdata.I_CoilSections[idxQuench_SIM, 0])
+        ## QL Integral 2  - from tPC
         ## QL Measured Data
         self.TDMSdata.QL2 = self.__calculateQuenchIntegral(self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_PC:],
                                                            self.TDMSdata.I_DCCT_MF[self.TDMSdata.trigger_PC:])
@@ -595,9 +623,7 @@ class QuenchPlanAnalysis():
         return
 
     def FindQuenchTDMS(self, Vthreshold, ValidationTime, Plot=1):
-        tSteps = int(ValidationTime/self.TDMSdata.t_d_MF)
-        found = 0
-
+        found = []
         if not self.MergedVoltages:
             print("Please first provide Coil-Structure. ")
             print("Obj.ProvideTurnsToCoilStructure(self, TurnsToVtaps: Dict)")
@@ -606,41 +632,79 @@ class QuenchPlanAnalysis():
         for key in self.MergedVoltages.keys():
             Udata = self.TDMSdata.groupMF[key].data
             if self.TDMSdata.trigger_shoot > 10000:
-                Udata = Udata - np.sum(Udata[:10000]) / 10000
+                offset = np.sum(Udata[:10000]) / 10000
+                Udata = Udata - offset
+                Vthr = Vthreshold
             else:
                 print("Failed to calculate SNR for Voltage channel.")
                 return
-            Qidx = np.where(abs(Udata[:self.TDMSdata.trigger_shoot+self.TDMSdata.trigger_PC]) > Vthreshold)[0]
+            Qidx = np.where(abs(Udata[:self.TDMSdata.trigger_shoot+self.TDMSdata.trigger_PC]) > Vthr)[0]
+            Qidx = Qidx[np.where(Qidx-self.TDMSdata.trigger_shoot > 0)[0]]
             if len(Qidx)> 0:
                 for i in range(len(Qidx)):
-                    QQidx = np.where(Udata[Qidx[i]:Qidx[i]+tSteps]>Vthreshold)[0]
+                    tSteps = int(ValidationTime / self.TDMSdata.t_d_MF)
+                    QQidx = np.where(abs(Udata[Qidx[i]:Qidx[i] + tSteps]) > Vthr)[0]
                     if len(QQidx) >= (0.95*tSteps) and (Qidx[i]+tSteps)<(self.TDMSdata.trigger_shoot+self.TDMSdata.trigger_PC):
-                        tQuench = Qidx[i]-self.TDMSdata.trigger_shoot
-                        if tQuench < 0:
-                            print("Detected Quench in {} at {}".format(key, (Qidx[i] - self.TDMSdata.trigger_shoot)*self.TDMSdata.t_d_MF))
-                        else:
-                            print("Detected Quench in {} at {}".format(key, self.TDMSdata.TimeFrame_MF[Qidx[i] - self.TDMSdata.trigger_shoot]))
-                        found = 1
+                        tQuench = self.TDMSdata.TimeFrame_MF[Qidx[i]-self.TDMSdata.trigger_shoot]
+                        idx_tQuench = Qidx[i]-self.TDMSdata.trigger_shoot
+                        found.append((key,  tQuench, idx_tQuench))
                         if Plot:
-                            try:
-                                fig = plt.figure()
-                                ax = fig.add_subplot(111)
-                                ax.grid(True)
-                                ax.set_title("Detected quench in - "+key)
-                                ax.set_ylim([-3*Vthreshold,3*Vthreshold])
-                                ax.plot(self.TDMSdata.TimeFrame_MF[tQuench-tSteps:tQuench+int(3*tSteps)],Udata[Qidx[i]-tSteps:Qidx[i]+int(3*tSteps)],'--')
-                                ax.axvline(self.TDMSdata.TimeFrame_MF[Qidx[i] - self.TDMSdata.trigger_shoot],c='r')
-                                ax.legend(["$U_{Res,"+key+"}$","$t_{Quench}$"])
-                            except:
-                                pass
+                            # try:
+                            fig = plt.figure()
+                            ax = fig.add_subplot(111)
+                            ax.grid(True)
+                            ax.set_title("Detected quench in - "+key)
+                            ax.set_ylim([-3*Vthr,3*Vthr])
+                            ax.set_xlabel("Time, t [ms]")
+                            ax.set_ylabel("Voltage [V]")
+                            if idx_tQuench-tSteps<0: tSteps = idx_tQuench
+                            ax.plot(self.TDMSdata.TimeFrame_MF[idx_tQuench-tSteps:idx_tQuench+int(5*tSteps)]*1000, Udata[Qidx[i]-tSteps:Qidx[i]+int(5*tSteps)],'--')
+                            ax.axvline(self.TDMSdata.TimeFrame_MF[idx_tQuench]*1000,c='r')
+                            ax.axhline(Vthr, c='r', ls='--')
+                            ax.legend(["$U_{Res,"+key+"}$","$t_{Quench}$","$U_{Threshold}$"])
+                            # except:
+                            #     pass
                         break
-
             # fig = plt.figure()
             # ax = fig.add_subplot(111)
             # ax.set_title(key)
             # ax.plot(np.linspace(0, len(Udata), len(Udata)), Udata, '--')
+            # ax.set_xlim([self.TDMSdata.trigger_shoot-50,self.TDMSdata.trigger_shoot+50+self.TDMSdata.trigger_PC])
+            # ax.set_ylim([-5*Vthr, 5*Vthr])
         if not found:
             print("No quench detected.")
+            found.append(("NaN",0,self.TDMSdata.trigger_PC))
+        else:
+            def getSecond(elem):
+                return elem[1]
+            found.sort(key=getSecond)
+            print("*** Ordered detected Quenches ***")
+            for j in range(len(found)):
+                print("Detected Quench in {} at {} ms".format(found[j][0], np.round(found[j][1] * 1000, 2)))
+
+            if Plot:
+                fig = plt.figure()
+                ax = fig.add_subplot(111)
+                count = 0
+                legend = []
+                for j in range(len(found)):
+                    key = found[j][0]
+                    idxTap = self.MergedVoltages[key]
+                    tx = str(count) + ": " + key + " at " + str(np.round(found[j][1] * 1000, 2)) + " ms"
+                    legend.append(tx)
+                    cmap = cm.get_cmap('winter', len(found))
+                    c = cmap(abs(found[j][1]*1000) / abs(found[0][1]*1000))
+                    c = np.array([c])
+                    pos = ax.scatter(self.SIMdata.XY_mag_ave[0, idxTap.astype(int) - 1],
+                               self.SIMdata.XY_mag_ave[1, idxTap.astype(int) - 1], s=10, c=c)
+                    count = count + 1
+                ax.axis('equal')
+                ax.grid('minor')
+                ax.set_xlabel("x [mm]")
+                ax.set_ylabel("y [mm]")
+                ax.set_title("Detected Quenches")
+                ax.legend(legend)
+        self.TDMSdata.Quenches = np.array(found)
 
     def GetCoilStructure(self, Print = 0, Plot = 1):
         if Print:
@@ -708,9 +772,9 @@ class QuenchPlanAnalysis():
             ax.grid(True)
             ax.set_ylabel("Current [A]", fontsize=20)
             ax.set_xlabel("Time [s]", fontsize=20)
-            ax.set_ylim([np.amin([np.amin(self.SIMdata.I_CoilSections),np.amin(self.SIMdata.I_CLIQ), np.amin(self.TDMSdata.I_DCCT_MF)])*1.1,
-                         np.amax([np.amax(self.SIMdata.I_CoilSections),np.amax(self.SIMdata.I_CLIQ), np.amax(self.TDMSdata.I_DCCT_MF)])*1.1])
-            ax.set_xlim([0, np.amax([np.amax(self.TDMSdata.TimeFrame_MF), np.amax(self.SIMdata.TimeFrame)])])
+            #ax.set_ylim([np.amin([np.amin(self.SIMdata.I_CoilSections),np.amin(self.SIMdata.I_CLIQ), np.amin(self.TDMSdata.I_DCCT_MF)])*1.1,
+            #             np.amax([np.amax(self.SIMdata.I_CoilSections),np.amax(self.SIMdata.I_CLIQ), np.amax(self.TDMSdata.I_DCCT_MF)])*1.1])
+            #ax.set_xlim([0, np.amax([np.amax(self.TDMSdata.TimeFrame_MF), np.amax(self.SIMdata.TimeFrame)])])
             ax.legend(legend,fontsize=20)
             ax.set_title(self.TDMSdata.FileName[-13:])
         return [self.TDMSdata, self.SIMdata]
\ No newline at end of file
