@@ -31,8 +31,10 @@ class ParametersCOSIM:
             print("No Magnet-Name defined. Setting to _0")
             self.nameMagnet = "_0"
         self.circuitName = nameCircuit
-        self.nameMagnet = nameMagnet
+        if type(nameMagnet) != list: self.nameMagnet = [nameMagnet]
+        else: self.nameMagnet = nameMagnet
         self.executionOrder = []
+        self.SimulationNumber = []
 
         # Load and set the default config files (using ResourceReader allows reading from a "hidden" resource folder)
         self.nameTemplateConfigFileCosim = ResourceReader.getResourcePath(os.path.join('sing', 'STEAMConfig.json'))
@@ -66,24 +68,24 @@ class ParametersCOSIM:
                 os.mkdir(nameFolderLedetModel)
             if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET')):
                 os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET'))
-            if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet)):
-                os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet))
-            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Input")):
-                os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Input"))
-            if not os.path.isdir(os.path.join(nameFolderLedetModel,"LEDET", self.nameMagnet, "Input","Initialize variables")):
-                os.mkdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Initialize variables"))
-                f = open(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Initialize variables", " .gitkeep"), "w")
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet[0])):
+                os.mkdir(os.path.join(nameFolderLedetModel, 'LEDET', self.nameMagnet[0]))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[0], "Input")):
+                os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[0], "Input"))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel,"LEDET", self.nameMagnet[0], "Input","Initialize variables")):
+                os.mkdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet[0],"Input","Initialize variables"))
+                f = open(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet[0],"Input","Initialize variables", " .gitkeep"), "w")
                 f.close()
-            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Input","Control current input")):
-                os.mkdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Control current input"))
-                f = open(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet,"Input","Control current input", " .gitkeep"), "w")
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[0], "Input","Control current input")):
+                os.mkdir(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet[0],"Input","Control current input"))
+                f = open(os.path.join(nameFolderLedetModel,"LEDET",self.nameMagnet[0],"Input","Control current input", " .gitkeep"), "w")
                 f.close()
             if not os.path.isdir(os.path.join(nameFolderLedetModel , "Field maps")):
                 os.mkdir(os.path.join(nameFolderLedetModel,"Field maps"))
-            if not os.path.isdir(os.path.join(nameFolderLedetModel, "Field maps", self.nameMagnet)):
-                os.mkdir(os.path.join(nameFolderLedetModel, "Field maps", self.nameMagnet))
-            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet ,"Output")):
-                os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet, "Output"))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "Field maps", self.nameMagnet[0])):
+                os.mkdir(os.path.join(nameFolderLedetModel, "Field maps", self.nameMagnet[0]))
+            if not os.path.isdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[0] ,"Output")):
+                os.mkdir(os.path.join(nameFolderLedetModel, "LEDET", self.nameMagnet[0], "Output"))
 
         if N_LEDET > 1:
             for i in range(1, N_LEDET+1):
@@ -183,6 +185,8 @@ class ParametersCOSIM:
                                t_0 = [0, 2e-5], t_end = [2e-5, 0.5], t_step_max = [[1e-5, 1e-5], [1e-5, 1e-5]],
                                relTolerance = [1e-4, None], absTolerance = [1, None], executionOrder = [1, 2],
                                executeCleanRun = [True, True], N_LEDET = 1, QuenchMagnet = 0, SimulationNumber = [0], DistinctMagnets = 1, PSPICEinitialConditions = []):
+        if len(SimulationNumber) < N_LEDET: self.SimulationNumber = SimulationNumber + (N_LEDET - len(SimulationNumber))*[0]
+        else: self.SimulationNumber = SimulationNumber
         # Components which are used in the Ports
         Components = ["L", "CoilSections"]  # [0] = PSpice, [1]=LEDET
         # Setting the Coilsections
@@ -268,7 +272,7 @@ class ParametersCOSIM:
             with open(filename, 'r') as f:
                 data = json.load(f)
                 data["solverPath"] = LEDETExecutable
-                data["modelName"] = self.nameMagnet
+                data["modelName"] = self.nameMagnet[0]
                 data["simulationNumber"] = str(SimulationNumber[0])
             try:
                 os.remove(filename)
@@ -457,8 +461,6 @@ class ParametersCOSIM:
                 json.dump(struct_total[i], f, indent=4)
                 f.write('\n')
 
-        print("I/O done!")
-
     def FindLibPath(self, File):
         path = os.path.abspath(os.path.join(os.getcwd(), os.pardir))
         for i in range(5):
@@ -496,6 +498,8 @@ class ParametersCOSIM:
         LEDETFiles = os.path.join(LEDETFiles, "steam-ledet-input")
         nameFolderCosimModel = self.nameFolderCosimModel
 
+        if type(nameMagnet) != list: nameMagnet = [nameMagnet]
+
         # Copy PSPICE model file
         if len(ManuallyStimulusFile) == 0:
             StimulusFile = self.FindLibPath(StimulusFile)
@@ -509,7 +513,6 @@ class ParametersCOSIM:
         self.CopyCircuit(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
         # makeCopyFile(nameFileSING, os.path.join(nameFolderPSpiceModel, 'Circuit.cir'))
         makeCopyFile(StimulusFile, os.path.join(nameFolderPSpiceModel, 'ExternalStimulus.stl'))
-        print("Copy PSPICE done!")
 
         LEDET_Fs = []
         for i in range(N_LEDET):
@@ -522,18 +525,20 @@ class ParametersCOSIM:
                     sourcedir = os.path.join(LEDETFiles, nameMagnet)
                     nM = nameMagnet
             else:
-                sourcedir = os.path.join(LEDETFiles, nameMagnet)
+                sourcedir = os.path.join(LEDETFiles, nameMagnet[0])
                 nameFolderLedetModel = os.path.join(nameFolderCosimModel, 'LEDET')
-                nM = nameMagnet
+                nM = nameMagnet[0]
             sourcefiles = os.listdir(sourcedir)
             if not os.path.isdir(nameFolderLedetModel):
                 os.mkdir(nameFolderLedetModel)
             destinationpath_field = nameFolderLedetModel + "//Field maps//" + nM
             destinationpath_para = nameFolderLedetModel + "//LEDET//" + nM + "//Input//"
+
+            SimNumber = self.SimulationNumber[i]
             for file in sourcefiles:
                 if file.endswith('.map2d') and not file.startswith(".sys"):
                     makeCopyFile(os.path.join(sourcedir, file), os.path.join(destinationpath_field, file))
-                if file.endswith('.xlsx') and not file.startswith(".sys") and file.startswith(str(nM)):
+                if file.endswith(str(SimNumber) +'.xlsx') and not file.startswith(".sys") and file.startswith(str(nM)):
                     shutil.copy(os.path.join(sourcedir, file), os.path.join(destinationpath_para, file))
                     LEDET_Fs.append(os.path.join(destinationpath_para, file))
                 if file.endswith('_selfMutualInductanceMatrix.csv'):
@@ -544,11 +549,24 @@ class ParametersCOSIM:
         TurnNumber = 0
         steps = 0
 
+        ## Count most frequent execution order
+        counter = 0
+        num = self.executionOrder[0]
+        for i in self.executionOrder:
+            curr_frequency = self.executionOrder.count(i)
+            if (curr_frequency > counter):
+                counter = curr_frequency
+                num = i
+
+        # If num ==1 change to 2 as 1 is by convention PSPICE
+        if num == 1: num = 2
+
+        # Prepare the files based on single function
         if type(files) == list:
             for i in range(len(files)):
                 file = files[i]
                 [Tnew, Snew] = self.prepareSingleLEDETFile(file)
-                if self.executionOrder[i+1] == self.executionOrder[i]:
+                if self.executionOrder[i+1] == num:
                     TurnNumber = TurnNumber + Tnew
                     steps = steps + Snew
             TurnNumber = int(TurnNumber / len(files))
@@ -578,6 +596,6 @@ class ParametersCOSIM:
         tV = a.Options.time_vector_params
         steps = 0
         for i in range(0,len(tV),3):
-            steps = steps+ (abs(tV[i])-abs(tV[i+2]))/tV[i+1]
+            steps = steps + abs(abs(tV[i])-abs(tV[i+2]))/tV[i+1]
 
         return [TurnNumber, steps]
